/*!
\file selector.cpp
\brief Class definition for Selector
\author Nicholas Phillips, phillips@friday.gsfc.nasa.gov
\date Feb 23, 2004
*/
#include <iostream>
#include <iomanip>

#include "selector.h"

using namespace std;
using namespace qglviewer;

Selector::Selector() : current_pix(-1)
{
	dc = 0.1;
	lower_lim = 0.5;
	direction = 1.0;
	color = lower_lim;
}


Selector::~Selector()
{
}

//==============================================================
bool Selector::select(const int x, const int y, Camera *camera, Projection *projection)
{
	Vec o,d;
	camera->convertClickToLine(x, y, o, d);
	selected = projection->select(o,d,v);
	return selected;
}

//==============================================================
void Selector::drawSphere()
{
	const float r = 0.99;
	int sr = 25;
	int nphi = 2*sr;
	int ntheta = sr;

	float dphi = 2*M_PI/(nphi-1);
	float dtheta = M_PI/(ntheta-1);

	float theta,phi;
	float x,y;
	float z0,r0;
	float z1,r1;

	theta = 0;
	z0 = r*cos(theta);
	r0 = r*sin(theta);
	glColor4f(color,color,color,1);
	for(int j = 1; j < ntheta; j++) {
		theta += dtheta;
		z1 = r*cos(theta);
		r1 = r*sin(theta);

		phi = 0;
		glBegin(GL_QUAD_STRIP);
		for(int i = 0; i < nphi; i++) {
			x = cos(phi);
			y = sin(phi);
			glVertex3f( r1*x, r1*y, z1);
			glVertex3f( r0*x, r0*y, z0);
			phi += dphi;
		}
		glEnd();

		z0 = z1;
		r0 = r1;
	}

	return;
}
//==============================================================
void Selector::draw()
{
	if( ! selected )
		return;

	drawSphere();
	/*
	glPointSize(3.0);
	glColor4f(1,0,0,1);
	glBegin( GL_POINTS );
	glVertex3fv( v.address());
	glEnd();
	*/
    return;
}
//==============================================================
void Selector::animate()
{
	if( ! selected )
		return;
	color += direction*dc;
	if( color > 1.0) {
		color = 1.0;
		direction *= -1;
	}
	else if( color < lower_lim) {
		color = lower_lim;
		direction *= -1;
	}

    return;
}


//==============================================================
void Selector::clear()
{
	selected = false;
	current_pix = -1;
	return;
}


//==============================================================
long Selector::toHealpix(const Skymap & skymap)
{
 	if( ! selected ) {
		current_pix = -1;
		return selectedPixel();
	}

	double theta = acos(v.z);
	double phi = atan2(v.y, v.x);
	/*
	cout << "theta: " << setw(12) << (180/M_PI)*theta
		 << ", phi : " << setw(12) << (180/M_PI)*phi;
	*/
	skymap.ang2pix(theta,phi,&current_pix);
	//cout << ", pix = " << current_pix << endl;

	return selectedPixel();
}
