/*!
\file skytexture.h
\brief Class declaration for SkyTexture
\author Nicholas Phillips, phillips@friday.gsfc.nasa.gov
\date Sep 24, 2004
*/
#ifndef SKYTEXTURE_H
#define SKYTEXTURE_H

#include "typedefs.h"
/**
Manage the OpenGL texture representation of the current skymap

@author Nicholas Phillips
*/
class SkyTexture{
private:
	unsigned char hilite_level;
	unsigned char select_level;
public:
    SkyTexture();

    virtual ~SkyTexture();
    void set(const int nside_, const bool ordering, const ColorTable &skycolor);
    void glTexture();
    void highlight(const int pixnum, const bool on);
    void select(const int pixnum, const bool on);

private:
	PixLUTCache::iterator getLUT(const int nside, const bool ordering);
	bool buildLUT(const int nside, const bool ordering);
private:
	unsigned char *texture;		//!< Texture representation of skymap
	int texture_res;			//!< Resolution per side of texture
	int nside;					//!< HealPix resolution parameter for texture
	PixLUT *lut;				//!< Pointer to current look-up table
	PixLUTCache lut_cache_ring;	//!< Cache of ring-ordered look-up tables
	PixLUTCache lut_cache_nest;	//!< Cache of nest-ordered look-up tables
};

#endif
